<?php

namespace EventPortal\Concerns;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Util\Status;
use EventPortal\Enums\InertiaMethod;
use Illuminate\Database\Eloquent\Builder;

trait HasStatus
{
    public function getReadableStatusAttribute(): string
    {
        return Status::make($this::KEY)->getReadable($this->attributes['status']);
    }

    public function getTransitionActions(): array
    {
        return collect($this->stateMachine()->getPossibleTransitions())
            ->map(function ($transition) {
                $parameters = array_merge($this->transitionRouteParameters(), [
                    $this::KEY => $this,
                    'type' => $transition,
                ]);

                return Button::make($this->stateMachine()->metadata()->transition($transition, 'title'))
                    ->method(InertiaMethod::POST())
                    ->route($this->transitionRouteName(), $parameters);
            })
            ->toArray();
    }

    public function scopeIsArchived(Builder $query): Builder
    {
        return $query->where('status', 'archived');
    }

    public function scopeIsNotArchived(Builder $query): Builder
    {
        return $query->where('status', '!=', 'archived');
    }
}
